@@ -55,7 +55,8 @@ bool network_bss_list_isempty(struct network *network);
 void network_bss_list_clear(struct network *network);
 struct scan_bss *network_bss_find_by_addr(struct network *network,
 							const uint8_t *addr);
-struct scan_bss *network_bss_select(struct network *network);
+struct scan_bss *network_bss_select(struct network *network,
+					bool fallback_to_blacklist);
 
 bool network_register(struct network *network, const char *path);
 
@@ -67,13 +68,17 @@ void network_rank_update(struct network *network, bool connected);
 void network_connect_new_hidden_network(struct network *network,
 						struct l_dbus_message *message);
 
+void network_blacklist_add(struct network *network, struct scan_bss *bss);
+
 struct network_info {
 	char ssid[33];
 	enum security type;
+	struct l_queue *known_frequencies;
 	struct timespec connected_time;	/* Time last connected */
 	int seen_count;			/* Ref count for network.info */
 	bool is_hidden:1;
 	bool is_known:1;
+	bool is_autoconnectable:1;
 };
 
 struct network_info *network_info_add_known(const char *ssid,
