@@ -28,10 +28,6 @@
 
 #define align_len(len, boundary) (((len)+(boundary)-1) & ~((boundary)-1))
 
-#define container_of(ptr, type, member) ({			\
-	const __typeof__( ((type *)0)->member ) *__mptr = (ptr);	\
-	(type *)( (char *)__mptr - offsetof(type,member) );})
-
 #define MAC "%02x:%02x:%02x:%02x:%02x:%02x"
 #define MAC_STR(a) a[0], a[1], a[2], a[3], a[4], a[5]
 
@@ -43,6 +39,9 @@ bool util_string_to_address(const char *str, uint8_t *addr);
 bool util_is_group_address(const uint8_t *addr);
 bool util_is_broadcast_address(const uint8_t *addr);
 
+const char *util_get_domain(const char *identity);
+const char *util_get_username(const char *identity);
+
 static inline uint8_t util_bit_field(const uint8_t oct, int start, int num)
 {
 	unsigned char mask = (1 << num) - 1;
