@@ -25,6 +25,7 @@
 
 struct wiphy;
 struct scan_bss;
+struct scan_freq_set;
 
 enum wiphy_state_watch_event {
 	WIPHY_STATE_WATCH_EVENT_POWERED,
@@ -39,7 +40,8 @@ typedef void (*wiphy_destroy_func_t)(void *user_data);
 enum ie_rsn_cipher_suite wiphy_select_cipher(struct wiphy *wiphy,
 							uint16_t mask);
 enum ie_rsn_akm_suite wiphy_select_akm(struct wiphy *wiphy,
-					struct scan_bss *bss);
+					struct scan_bss *bss,
+					bool fils_capable_hint);
 
 bool wiphy_parse_id_and_name(struct l_genl_attr *attr, uint32_t *out_id,
 				const char **out_name);
@@ -47,9 +49,13 @@ bool wiphy_parse_id_and_name(struct l_genl_attr *attr, uint32_t *out_id,
 struct wiphy *wiphy_find(int wiphy_id);
 
 struct wiphy *wiphy_create(uint32_t wiphy_id, const char *name);
+void wiphy_create_complete(struct wiphy *wiphy);
 bool wiphy_destroy(struct wiphy *wiphy);
 void wiphy_update_from_genl(struct wiphy *wiphy, struct l_genl_msg *msg);
 
+bool wiphy_constrain_freq_set(const struct wiphy *wiphy,
+						struct scan_freq_set *set);
+
 const char *wiphy_get_path(struct wiphy *wiphy);
 uint32_t wiphy_get_supported_bands(struct wiphy *wiphy);
 const struct scan_freq_set *wiphy_get_supported_freqs(
@@ -61,6 +67,7 @@ bool wiphy_has_ext_feature(struct wiphy *wiphy, uint32_t feature);
 uint8_t wiphy_get_max_num_ssids_per_scan(struct wiphy *wiphy);
 bool wiphy_supports_iftype(struct wiphy *wiphy, uint32_t iftype);
 bool wiphy_supports_adhoc_rsn(struct wiphy *wiphy);
+const char *wiphy_get_driver(struct wiphy *wiphy);
 
 uint32_t wiphy_state_watch_add(struct wiphy *wiphy,
 				wiphy_state_watch_func_t func, void *user_data,
